@@ -73,6 +73,8 @@
 #include <sys/time.h>
 #include <getopt.h> /* GNU extension: long options */
 #include <sched.h>
+#include <locale.h>
+#include <libintl.h>
 
 /* For debugging purposes */
 #include <mcheck.h>
@@ -119,6 +121,13 @@ int main(int argc, char *argv[]) {
   debug = DEBUG;
 #endif
 
+  /* prepare for i18n */
+  setlocale(LC_ALL, "");
+  if (!textdomain(PACKAGE)) {
+    fprintf(stderr, "Error setting domainname for gettext() "
+	            "(internationalization).\n");
+  }
+
   /* only useful for yet to code separate audio thread
      (rt only for audio thread!)
   struct sched_param schedpar; // scheduler structure for priority
