@@ -5,6 +5,20 @@
  *
  * Copyright 2002, 2003 Roland Stigge
  *
+ * ANT is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ANT is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ANT; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
  */
 
 #include "config.h"
@@ -32,10 +46,6 @@
 #include "in.xpm"
 #include "out.xpm"
 
-struct cid_row_t { /* _additional_ (invisible) data associated with a row */
-  int marked_unanswered;
-};
-
 /*
  * returns a new data structure (for association with a row)
  * filled with default values
@@ -59,7 +69,7 @@ static void cid_row_destroy(gpointer rowdata) {
 
 /* called when caller id monitor state check button is toggled */
 void cid_toggle_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
 
   if (GTK_CHECK_MENU_ITEM (session->cid_check_menu_item)->active) {
     gtk_widget_show(session->cid);
@@ -73,25 +83,16 @@ void cid_toggle_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
 /*
  * called on cid delete request
  *
- * -> deletes entry without confirmation
- *
- *  input:  widget["to_delete"]: number of cid_list item to delete
- *          data: session
- *  
- *  output: widget["to_delete"] entry will be removed
+ * -> deletes specified row without confirmation
  */
-static void cid_request_delete(GtkWidget *widget, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
-  int row;
-
-  if ((row = (int)gtk_object_get_data(GTK_OBJECT(widget), "to_delete"))) {
-    cid_mark_row(session, row, 0);
-    session->cid_num--;
-    /* decrease before removal: select-row signal submission */
-    gtk_clist_remove(GTK_CLIST(session->cid_list), row);
+static void cid_request_delete(GtkWidget *widget _U_, gpointer data,
+                               guint row) {
+  session_t *session = (session_t *) data;
   
-    gtk_object_remove_data(GTK_OBJECT(widget), "to_delete");
-  }
+  cid_mark_row(session, row, FALSE);
+  session->cid_num--;
+  /* decrease before removal: "select-row" signal submission -> other cb */
+  gtk_clist_remove(GTK_CLIST(session->cid_list), row);
 }
 
 /*
@@ -103,10 +104,9 @@ static gint cid_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data) {
     case GDK_Delete:     /* Delete dialog */
     case GDK_KP_Delete:
       /* portability: GtkCList.selection is private! */
-      if (GTK_CLIST(((struct session_t *) data)->cid_list)->selection) {
-        gtk_object_set_data(GTK_OBJECT(widget), "to_delete",
-	    GTK_CLIST(((struct session_t *) data)->cid_list)->selection->data);
-        cid_request_delete(widget, data);
+      if (GTK_CLIST(((session_t *) data)->cid_list)->selection) {
+        cid_request_delete(widget, data,
+	    (int)GTK_CLIST(((session_t *) data)->cid_list)->selection->data);
       }
       return TRUE; /* event handled */
     default:
@@ -119,27 +119,51 @@ static gint cid_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data) {
  */
 static gint cid_mouse_cb(GtkWidget *widget _U_,
                          GdkEventButton *event, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   
   if (event->button == 3) { /* popup menu */
-    GtkMenu *menu = GTK_MENU(gtk_menu_new());
-    GtkWidget *item = gtk_menu_item_new_with_label("Delete");
     gint row;
     gint column;
 
     if (gtk_clist_get_selection_info(GTK_CLIST(session->cid_list),
 	                             event->x, event->y, &row, &column)) {
 
-      gtk_object_set_data(GTK_OBJECT(item), "to_delete", (gpointer) row);
-      gtk_signal_connect(GTK_OBJECT(item), "activate",
-                         GTK_SIGNAL_FUNC(cid_request_delete),
-		         data);
-      gtk_widget_show(item);
-      gtk_menu_append(menu, item);
+      GtkItemFactoryEntry menu_items[] = {
+        /* path                 accel. callb.         cb param. kind         */
+        { "/_Playback",         NULL,  NULL,          0,        ""           },
+        { "/Delete _Recording", NULL,  NULL,          0,        ""           },
+        { "/Sep",               NULL,  NULL,          0,        "<Separator>"},
+        { "/_Delete Row",       NULL,  cid_request_delete, row, ""           }
+      };
+      GtkMenu *menu = GTK_MENU(gtk_menu_new());
+      GtkItemFactory *item_factory;
+      GtkAccelGroup *accel_group;
+
+      gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
+
+      GtkWidget *playback_item;
+      GtkWidget *delete_record_item;
+      GtkWidget *delete_item;
+  
+      accel_group = gtk_accel_group_new();
+      item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<popup>",
+				      accel_group);
+      gtk_item_factory_create_items_ac(item_factory, nmenu_items, menu_items,
+				   session, 2);
+      playback_item = gtk_item_factory_get_item(item_factory, "/Playback");
+      delete_record_item =
+	gtk_item_factory_get_item(item_factory, "/Delete Recording");
+      delete_item = gtk_item_factory_get_item(item_factory, "/Delete Row");
+ 
+      gtk_widget_set_sensitive(playback_item, FALSE);
+      gtk_widget_set_sensitive(delete_record_item, FALSE);
+  
+      menu = GTK_MENU(gtk_item_factory_get_widget(item_factory, "<popup>"));
+      gtk_menu_set_accel_group(menu, accel_group);
 
       gtk_menu_popup(menu, NULL, NULL, NULL, NULL, event->button, event->time);
     }
-    return TRUE;
+    return TRUE; /* event handled */
   } else {
     return FALSE;
   }
@@ -150,10 +174,10 @@ static gint cid_mouse_cb(GtkWidget *widget _U_,
  */
 static void cid_select_row_cb(GtkCList *list _U_, gint row, gint column _U_,
                               GdkEventButton *button _U_, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
 
   if (row < session->cid_num)
-    cid_mark_row(session, row, 0);
+    cid_mark_row(session, row, FALSE);
 }
 
 /*
@@ -161,15 +185,22 @@ static void cid_select_row_cb(GtkCList *list _U_, gint row, gint column _U_,
  *   sets CID related members in session
  * NOTE: caller has to gtk_widget_show() this widget
  */
-GtkWidget *cid_new(struct session_t *session) {
+GtkWidget *cid_new(session_t *session) {
   GtkWidget *frame;           /* frame with comment */
   GtkWidget *scrolled_window; /* the home for the clist */
   GtkWidget *clist;           /* the list itself */
   GtkStyle *style;            /* needed for preparing symbols */
 
-  gchar *titles[] = {"Date/Time", "Type", "From", "To", "Duration" /*XXX, "Zone" */};
-
+  gchar *titles[CID_COL_NUMBER];
   frame = gtk_frame_new("Caller ID");
+
+  titles[CID_COL_FLAGS   ] = "";
+  titles[CID_COL_TIME    ] = "Date/Time";
+  titles[CID_COL_TYPE    ] = "Type";
+  titles[CID_COL_FROM    ] = "From";
+  titles[CID_COL_TO      ] = "To";
+  titles[CID_COL_DURATION] = "Duration";
+  
   gtk_container_set_border_width(GTK_CONTAINER(frame), 8);
 
   /* the scrolled window: no special adjustments */
@@ -182,21 +213,22 @@ GtkWidget *cid_new(struct session_t *session) {
   gtk_widget_show(scrolled_window);
 
   /* the list widget */
-  clist = gtk_clist_new_with_titles(sizeof(titles) / sizeof(gchar *), titles);
+  clist = gtk_clist_new_with_titles(CID_COL_NUMBER, titles);
   
   /* default: GTK_SELECTION_MULTIPLE: */
   gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
 
   gtk_clist_set_shadow_type(GTK_CLIST(clist), GTK_SHADOW_IN);
-  gtk_clist_set_column_width(GTK_CLIST(clist), 0, 128);
-  gtk_clist_set_column_width(GTK_CLIST(clist), 1, 30);
-  gtk_clist_set_column_width(GTK_CLIST(clist), 2, 100);
-  gtk_clist_set_column_width(GTK_CLIST(clist), 3, 100);
-  gtk_clist_set_column_width(GTK_CLIST(clist), 4, 80);
+  gtk_clist_set_column_width(GTK_CLIST(clist), CID_COL_FLAGS, 14);
+  gtk_clist_set_column_width(GTK_CLIST(clist), CID_COL_TIME, 128);
+  gtk_clist_set_column_width(GTK_CLIST(clist), CID_COL_TYPE, 30);
+  gtk_clist_set_column_width(GTK_CLIST(clist), CID_COL_FROM, 100);
+  gtk_clist_set_column_width(GTK_CLIST(clist), CID_COL_TO, 100);
+  gtk_clist_set_column_width(GTK_CLIST(clist), CID_COL_DURATION, 80);
   /*gtk_clist_set_column_width(GTK_CLIST(clist), 5, 80);*/
   gtk_container_add(GTK_CONTAINER(scrolled_window), clist);
   gtk_clist_column_titles_show(GTK_CLIST(clist));
-  gtk_widget_set_usize(clist, 480, 128);
+  gtk_widget_set_usize(clist, 500, 128);
 
   gtk_signal_connect(GTK_OBJECT(clist), "key-press-event",
                      GTK_SIGNAL_FUNC(cid_key_cb), session);
@@ -229,7 +261,7 @@ GtkWidget *cid_new(struct session_t *session) {
 /*
  * select last item in cid list and adjust view to end of list
  */
-void cid_jump_to_end(struct session_t *session) {
+void cid_jump_to_end(session_t *session) {
   GtkAdjustment *adj;
 
   gtk_clist_select_row(GTK_CLIST(session->cid_list),
@@ -243,7 +275,7 @@ void cid_jump_to_end(struct session_t *session) {
 /*
  * remove unneeded entries from caller id list (shorten log)
  */
-void cid_normalize(struct session_t *session) {
+void cid_normalize(session_t *session) {
   while (session->cid_num_max && session->cid_num > session->cid_num_max) {
     gtk_clist_remove(GTK_CLIST(session->cid_list), 0);
     session->cid_num--;
@@ -273,25 +305,27 @@ static char *cid_timestring(time_t t) {
  *
  * from == NULL, if still unknown
  */
-void cid_add_line(struct session_t *session,
+void cid_add_line(session_t *session,
 		  enum call_type_t ct, gchar *from, gchar *to) {
-  gchar *line[5];
+  gchar *line[CID_COL_NUMBER];
   char *typestring;
 
   GdkPixmap *pixmap;
   GdkBitmap *bitmap;
  
   /* First: date */
-  line[0] = cid_timestring(session->ring_time);
-  line[1] = ""; /* call type */
+  line[CID_COL_TIME] = cid_timestring(session->ring_time);
+
+  line[CID_COL_FLAGS] = "";
+  line[CID_COL_TYPE] = ""; /* call type */
 
   if (from == NULL) /* the from field */
-    line[2] = "???";
+    line[CID_COL_FROM] = "???";
   else
-    line[2] = from;
-  line[3] = to;
+    line[CID_COL_FROM] = from;
+  line[CID_COL_TO] = to;
 
-  line[4] = "???"; /* duration */
+  line[CID_COL_DURATION] = "???"; /* duration */
 
   /* create line */
   session->cid_num = gtk_clist_append(GTK_CLIST(session->cid_list), line) + 1;
@@ -311,23 +345,24 @@ void cid_add_line(struct session_t *session,
     bitmap = session->symbol_out_bitmap;
     typestring = "OUT";
   }
-  gtk_clist_set_pixtext(GTK_CLIST(session->cid_list), session->cid_num - 1 , 1,
-			typestring, 50, pixmap, bitmap);
+  gtk_clist_set_pixtext(GTK_CLIST(session->cid_list), session->cid_num - 1 ,
+                        CID_COL_TYPE, typestring, 50, pixmap, bitmap);
 
   cid_jump_to_end(session);
   
   /* clean up */
-  free(line[0]);
+  free(line[CID_COL_TIME]);
 }
 
 /*
  * set "date" field in last line
  */
-void cid_set_date(struct session_t *session, time_t date) {
+void cid_set_date(session_t *session, time_t date) {
   if (date) {
     char *temp;
     gtk_clist_set_text(GTK_CLIST(session->cid_list),
-		       session->cid_num - 1, 0, temp = cid_timestring(date));
+	               session->cid_num - 1, CID_COL_TIME,
+		       temp = cid_timestring(date));
     free(temp);
   }
 }
@@ -335,10 +370,10 @@ void cid_set_date(struct session_t *session, time_t date) {
 /*
  * set "from" field in last line
  */
-void cid_set_from(struct session_t *session, gchar *from) {
+void cid_set_from(session_t *session, gchar *from) {
   if (from)
     gtk_clist_set_text(GTK_CLIST(session->cid_list),
-		       session->cid_num - 1, 2, from);
+		       session->cid_num - 1, CID_COL_FROM, from);
 }
 
 /*
@@ -347,22 +382,22 @@ void cid_set_from(struct session_t *session, gchar *from) {
  * if message == NULL, current time will be used to calculate a duration
  * if message != NULL, this string will be displayed in the Duration field (#4)
  */
-void cid_set_duration(struct session_t *session, gchar *message) {
+void cid_set_duration(session_t *session, gchar *message) {
   char *buf = timediff_str(time(NULL), session->vcon_time);
   
   if (message)
-    gtk_clist_set_text(GTK_CLIST(session->cid_list), session->cid_num - 1, 4,
-		       message);
+    gtk_clist_set_text(GTK_CLIST(session->cid_list), session->cid_num - 1,
+	               CID_COL_DURATION, message);
   else
-    gtk_clist_set_text(GTK_CLIST(session->cid_list), session->cid_num - 1, 4,
-		       buf);
+    gtk_clist_set_text(GTK_CLIST(session->cid_list), session->cid_num - 1,
+	               CID_COL_DURATION, buf);
   free(buf);
 }
 
 /*
  * Add a line to clist (e.g. from saved caller id file)
  */
-void cid_add_saved_line(struct session_t *session, char *date, char *type,
+void cid_add_saved_line(session_t *session, char *date, char *type,
 			char *from, char *to, char *duration) {
   if (debug > 1)
     fprintf(stderr, "Caller ID add:\n"
@@ -372,22 +407,22 @@ void cid_add_saved_line(struct session_t *session, char *date, char *type,
   cid_add_line(session,
 	       !strncmp(type, "IN", 2) ? CALL_IN : CALL_OUT, from, to);
   gtk_clist_set_text(GTK_CLIST(session->cid_list),
-		     session->cid_num - 1, 0, date);
+		     session->cid_num - 1, CID_COL_TIME, date);
   gtk_clist_set_text(GTK_CLIST(session->cid_list),
-		     session->cid_num - 1, 4, duration);
+		     session->cid_num - 1, CID_COL_DURATION, duration);
 }
 
 /*
  * Merges isdnlog data into existing cid list
  */
-void cid_calls_merge(struct session_t *session) {
+void cid_calls_merge(session_t *session) {
   int low, high, mid; /* bounds and helper index */
   time_t start = time(NULL)
                         - 60 * 60 * 24 * session->option_calls_merge_max_days;
   int callstime; /* time of calls file line */
   char *cidstr; /* time of cid_list entry */
   size_t linelength = 100; /* buffer size */ 
-  char *line = (char *) malloc (linelength); /* buffer for getline() */
+  char *line = (char *) malloc (linelength); /* initial buffer for getline() */
   FILE *f;
 
   char from[41]; /* data to read from calls file */
@@ -399,7 +434,7 @@ void cid_calls_merge(struct session_t *session) {
   char *temp;
   int rel;
   int cidline; /* index into cid list */
-  gchar *linearr[5]; /* line to insert to cid_list */
+  gchar *linearr[CID_COL_NUMBER]; /* line to insert to cid_list */
   GdkPixmap *pixmap;
   GdkBitmap *bitmap;
 
@@ -461,32 +496,33 @@ void cid_calls_merge(struct session_t *session) {
       }
       
       /* prepare (text) line to merge */
-      linearr[0] = cid_timestring((time_t)date);
-      linearr[1] = type;
-      if (from[0])
-	linearr[2] = from;
+      linearr[CID_COL_TIME] = cid_timestring((time_t)date);
+      linearr[CID_COL_TYPE] = type;
+      if (*from)
+	linearr[CID_COL_FROM] = from;
       else
-	linearr[2] = "???";
-      if (to[0])
-	linearr[3] = to;
+	linearr[CID_COL_FROM] = "???";
+      if (*to)
+	linearr[CID_COL_TO] = to;
       else
-	linearr[3] = "???";
+	linearr[CID_COL_TO] = "???";
       if (duration < 0) {
-	linearr[4] = strdup("(UNKNOWN)");
+	linearr[CID_COL_DURATION] = strdup("(UNKNOWN)");
       } else {
-	linearr[4] = timediff_str(duration, (time_t) 0);
+	linearr[CID_COL_DURATION] = timediff_str(duration, (time_t) 0);
       }
 
       /* where to merge it in? */
       rel = 1; /* init to merge by default */
       while (cidline < session->cid_num &&
-        (gtk_clist_get_text(GTK_CLIST(session->cid_list), cidline, 0, &cidstr),
-	 (rel = strcmp(cidstr, linearr[0])) < 0)) {
+        (gtk_clist_get_text(GTK_CLIST(session->cid_list),
+			    cidline, CID_COL_TIME, &cidstr),
+	 (rel = strcmp(cidstr, linearr[CID_COL_TIME])) < 0)) {
 	
 	cidline++;
       }
 
-      if ((rel > 0 && strcmp(linearr[0], cache) > 0)
+      if ((rel > 0 && strcmp(linearr[CID_COL_TIME], cache) > 0)
 	  || cidline == session->cid_num) {
 	/* only merge new and successive entries */
 	
@@ -494,15 +530,15 @@ void cid_calls_merge(struct session_t *session) {
         gtk_clist_set_row_data_full(GTK_CLIST(session->cid_list), cidline,
                                     cid_row_new(), cid_row_destroy);
         gtk_clist_set_pixtext(GTK_CLIST(session->cid_list),
-	    cidline , 1, type, 50, pixmap, bitmap);
+	    cidline, CID_COL_TYPE, type, 50, pixmap, bitmap);
 	session->cid_num++;
       }
       free(cache);
-      cache = linearr[0];
-      free(linearr[4]);
+      cache = linearr[CID_COL_TIME];
+      free(linearr[CID_COL_DURATION]);
     }
 
-    free(linearr[0]);
+    free(linearr[CID_COL_TIME]);
     
     cid_normalize(session);
   
@@ -523,7 +559,7 @@ void cid_calls_merge(struct session_t *session) {
  *        row     number of specified row
  *        state   1 for mark, 0 for unmark
  */
-void cid_mark_row(struct session_t *session, int row, int state) {
+void cid_mark_row(session_t *session, int row, int state) {
   struct cid_row_t *rowdata = (struct cid_row_t *) gtk_clist_get_row_data(
                                             GTK_CLIST(session->cid_list), row);
   GdkColor color;
