@@ -5,6 +5,20 @@
  *
  * Copyright 2002, 2003 Roland Stigge
  *
+ * ANT is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ANT is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ANT; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
  */
 
 /* regular GNU system includes */
@@ -39,7 +53,7 @@
  *        data = the session
  */
 static void controlpad_preset_edit_ok(GtkWidget *widget, gpointer *data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   GtkWidget *entry_name = gtk_object_get_data(GTK_OBJECT(widget),
                                                       "entry_name");
   GtkWidget *entry_number = gtk_object_get_data(GTK_OBJECT(widget),
@@ -68,7 +82,7 @@ static void controlpad_preset_edit_ok(GtkWidget *widget, gpointer *data) {
 static gint controlpad_preset_edit_cb(GtkWidget *widget, GdkEventButton *event,
                                       gpointer data) {
   if (event->button == 3) {
-    struct session_t *session = (struct session_t *) data;
+    session_t *session = (session_t *) data;
     char *buttoncode = (char *)gtk_object_get_data(GTK_OBJECT(widget), "desc");
     int n = strtol(&buttoncode[1], NULL, 0);
     GtkWidget *window;
@@ -155,7 +169,7 @@ static gint controlpad_preset_edit_cb(GtkWidget *widget, GdkEventButton *event,
  * called when button in key pad clicked (except mute)
  */
 static void controlpad_button_cb(GtkWidget *button, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   char *c = (char *) gtk_object_get_data(GTK_OBJECT(button), "desc");
   char *temp;
 
@@ -215,7 +229,7 @@ static void controlpad_button_cb(GtkWidget *button, gpointer data) {
  * called when mute button in key pad toggled
  */
 static void controlpad_mute_cb(GtkWidget *button, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) { /* muted */
     session->option_muted = 1;
@@ -230,7 +244,7 @@ static void controlpad_mute_cb(GtkWidget *button, gpointer data) {
  * called when record / local / remote checkbutton has been toggled
  */
 static void controlpad_record_cb(GtkWidget *button, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   char *digits = NULL;
 
   if (button == session->record_checkbutton) {
@@ -282,7 +296,7 @@ static void controlpad_record_cb(GtkWidget *button, gpointer data) {
 /*
  * The key pad etc.
  */
-GtkWidget *controlpad_new(struct session_t *session) {
+GtkWidget *controlpad_new(session_t *session) {
   char *labels[4][5] = {{"1", "2", "3", "B", session->preset_names[0]},
 			{"4", "5", "6", "C", session->preset_names[1]},
 			{"7", "8", "9", "R", session->preset_names[2]},
@@ -450,7 +464,7 @@ GtkWidget *controlpad_new(struct session_t *session) {
 /* called when control pad state check button is toggled */
 void controlpad_toggle_cb(GtkWidget *widget _U_,
 			  gpointer data, guint action _U_) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
 
   if (GTK_CHECK_MENU_ITEM (session->controlpad_check_menu_item)->active) {
     gtk_widget_show(session->controlpad);
