@@ -5,6 +5,20 @@
  *
  * Copyright 2002, 2003 Roland Stigge
  *
+ * ANT is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ANT is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ANT; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
  */
 
 /* regular GNU system includes */
@@ -33,7 +47,7 @@
  *          seconds: position of sample in seconds
  * returns: generated ulaw sample
  */
-unsigned char fxgenerate(struct session_t *session, enum effect_t effect,
+unsigned char fxgenerate(session_t *session, enum effect_t effect,
 			 int index, double seconds) {
   unsigned char x; /* generated ulaw sample (return value)     */
   double rest;     /* monitor the period in seconds modulo 4   */
