@@ -5,6 +5,20 @@
  *
  * Copyright 2002, 2003 Roland Stigge
  *
+ * ANT is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ANT is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ANT; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
  */
 
 #include "config.h"
@@ -20,6 +34,7 @@
 
 /* GTK */
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 
 /* own header files */
 #include "gtk.h"
@@ -51,7 +66,7 @@ volatile int interrupted = 0; /* the caught signal will be stored here */
  * input: data: session
  */
 static void quit(GtkWidget *widget _U_, gpointer data, guint action _U_) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   
   settings_history_write(session); /* write history */
   settings_callerid_write(session); /* write callerid history */
@@ -87,7 +102,7 @@ void terminate_signal_callback(int sig) {
  * periodically (e.g. each 300 milliseconds) from gtk main loop called function
  */
 gint timeout_callback(gpointer data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
 
   if (interrupted) {
     switch(interrupted) {
@@ -209,7 +224,7 @@ struct info_row_t {
 /* Info window */
 static void cb_info_window(GtkWidget *widget _U_, gpointer data,
 			   guint action _U_) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   int inactive = session->option_release_devices &&
     (session->state == STATE_READY || session->state == STATE_RINGING_QUIET);
   struct info_row_t rows[] = {
@@ -354,7 +369,7 @@ static void cb_license(GtkWidget *widget _U_, gpointer data _U_, guint action _U
  *
  * NOTE: assumes other parts to be initialized (maybe not shown already)
  */
-GtkWidget *get_main_menu(GtkWidget *window, struct session_t *session) {
+GtkWidget *get_main_menu(GtkWidget *window, session_t *session) {
   /* The main menu structure */
   GtkItemFactoryEntry main_menu_items[] = {
     /*path                  accel.    callb.         cb param. kind          */
@@ -378,7 +393,7 @@ GtkWidget *get_main_menu(GtkWidget *window, struct session_t *session) {
 
     { "/_Help",             NULL,     NULL,          0,        "<LastBranch>"},
     { "/Help/_About",       NULL,     cb_about,      0,        NULL          },
-    { "/Help/_License",     NULL,     cb_license,    0,        NULL          },
+    { "/Help/_License",     NULL,     cb_license,    0,        NULL          }
     /*        set to session by ..._create_items_ac -^                       */
   };
 
@@ -418,12 +433,29 @@ GtkWidget *get_main_menu(GtkWidget *window, struct session_t *session) {
   return gtk_item_factory_get_widget(item_factory, "<main>");
 }
 
+/*
+ * called on key pressed in combo box entry
+ */
+static gint entry_key_cb(GtkWidget *entry, GdkEventKey *event, gpointer data) {
+  session_t *session = (session_t *) data;
+
+  if (event->keyval == GDK_KP_Enter) { /* catch keyboard keypad Enter */
+    gtk_button_clicked(GTK_BUTTON(session->pick_up_button));
+    /* the keyboard keypad Enter generates an unneeded character, so
+       discard it: */
+    gtk_signal_emit_stop_by_name(GTK_OBJECT(entry), "key-press-event");
+    return TRUE; /* event handled */
+  } else {
+    return FALSE;
+  }
+}
+
 /*
  * Get the basic dial box with entry and dial / hang up buttons
  * sets dial box members in session
  * NOTE: caller has to gtk_widget_show it itself
  */
-GtkWidget *get_dial_box(struct session_t *session) {
+GtkWidget *get_dial_box(session_t *session) {
   GtkWidget *frame;
   GtkWidget *hbox;
   GtkWidget *label;
@@ -475,6 +507,9 @@ GtkWidget *get_dial_box(struct session_t *session) {
 			    "activate",
 			    GTK_SIGNAL_FUNC(gtk_button_clicked),
 			    GTK_OBJECT(session->pick_up_button));
+  /* handle special keys */
+  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(session->dial_number_box)->entry),
+                     "key-press-event", GTK_SIGNAL_FUNC(entry_key_cb), session);
 
   /* pick up button hbox */
   label_hbox = gtk_hbox_new(FALSE, 0);
@@ -542,7 +577,7 @@ GtkWidget *get_dial_box(struct session_t *session) {
  *
  * returns int to be returned from main()
  */
-int main_gtk(struct session_t *session) {
+int main_gtk(session_t *session) {
   GtkWidget *main_window;
   GtkWidget *main_vbox;
   GtkWidget *main_menu;
