@@ -36,7 +36,7 @@
  * returns 0 on success, -1 otherwise (bad audio settings)
  */
 static int gtksettings_try(GtkWidget *widget) {
-  struct session_t *session = gtk_object_get_data(GTK_OBJECT(widget),
+  session_t *session = gtk_object_get_data(GTK_OBJECT(widget),
 						  "session");
   /* save current settings */
   char *old_msn = session->msn;
@@ -235,7 +235,7 @@ static void gtksettings_cb_ok(GtkWidget *window) {
 
 /* clicked "Save" button at settings dialog (gtksettings_cb) */
 static void gtksettings_cb_save(GtkWidget *widget) {
-  struct session_t *session = gtk_object_get_data(GTK_OBJECT(widget),
+  session_t *session = gtk_object_get_data(GTK_OBJECT(widget),
 						  "session");
   gtksettings_try(widget);
   settings_options_write(session);
@@ -243,7 +243,7 @@ static void gtksettings_cb_save(GtkWidget *widget) {
 
 /* clicked "Cancel" button at settings dialog (gtksettings_cb) */
 static void gtksettings_cb_cancel(GtkWidget *window) {
-  struct session_t *session = gtk_object_get_data(GTK_OBJECT(window),
+  session_t *session = gtk_object_get_data(GTK_OBJECT(window),
 						  "session");
   if (session->audio_device_name_in) /* audio not shut off */
     gtk_widget_destroy(window);
@@ -259,7 +259,7 @@ static void toggle_sensitive_register(GtkToggleButton *button,
 
 /* Settings menu entry callback */
 void gtksettings_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   GtkWidget *window;   /* the dialog window itself */
   GtkWidget *button_box; /* box for buttons in standard way */
   GtkWidget *button;   /* action area buttons */
