@@ -5,6 +5,20 @@
  *
  * Copyright 2002, 2003 Roland Stigge
  *
+ * ANT is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ANT is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ANT; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
  */
 
 #include "config.h"
@@ -40,9 +54,9 @@
 /*
  * This is our session. Currently just one globally.
  */
-struct session_t session;
+session_t session;
 
-struct state_data_t state_data[] = {
+struct state_data_t state_data[STATE_NUMBER] = {
   { "Ready",          "Dial",    1, "Hang up", 0 }, /* STATE_READY         */
   { "RING",           "Answer",  1, "Reject",  1 }, /* STATE_RINGING       */
   { "RING",           "Answer",  1, "Reject",  1 }, /* STATE_RINGING_QUIET */
@@ -56,7 +70,7 @@ struct state_data_t state_data[] = {
  *
  * returns 0 on success, -1 on error
  */
-int session_audio_open(struct session_t *session) {
+int session_audio_open(session_t *session) {
   if (debug)
     fprintf(stderr, "session_audio_open: Opening audio device(s).\n");
   if (open_audio_devices(session->audio_device_name_in,
@@ -81,7 +95,7 @@ int session_audio_open(struct session_t *session) {
  *
  * returns 0 on success, -1 on error
  */
-int session_audio_close(struct session_t *session) {
+int session_audio_close(session_t *session) {
   if (debug)
     printf("session_audio_close: Closing audio device(s).\n");
   if (close_audio_devices(session->audio_fd_in, session->audio_fd_out)) {
@@ -100,7 +114,7 @@ int session_audio_close(struct session_t *session) {
  * output: on success, session->audio_LUT* and session->audio_*buf are
  *         allocated (and set, if appropriate)
  */
-int session_audio_init(struct session_t *session) {
+int session_audio_init(session_t *session) {
   /* message: open audio device(s) */
   if (debug) {
     if (strcmp(session->audio_device_name_in, session->audio_device_name_out))
@@ -166,7 +180,7 @@ int session_audio_init(struct session_t *session) {
  *
  * returns 0 on success, -1 otherwise
  */
-int session_isdn_init(struct session_t *session) {
+int session_isdn_init(session_t *session) {
   /* open and init isdn device */
   if (debug)
     fprintf(stderr, "Initializing ISDN device...\n");
@@ -208,7 +222,7 @@ int session_isdn_init(struct session_t *session) {
 /*
  * init recording related things in session
  */
-int session_recording_init(struct session_t *session) {
+int session_recording_init(session_t *session) {
   /* mediation recording stuff */
   session->rec_buf_local_size = MEDIATION_RECBUFSIZE;
   session->rec_buf_remote_size = MEDIATION_RECBUFSIZE;
@@ -234,7 +248,7 @@ int session_recording_init(struct session_t *session) {
 /*
  * deinit recording related things in session
  */
-int session_recording_deinit(struct session_t *session) {
+int session_recording_deinit(session_t *session) {
   free(session->rec_buf_local);
   free(session->rec_buf_remote);
   free(session->recorder);
@@ -246,7 +260,7 @@ int session_recording_deinit(struct session_t *session) {
  * -> prepares de-initialization
  * -> is part of session_audio_free()
  */
-void session_audio_free(struct session_t *session) {
+void session_audio_free(session_t *session) {
   free(session->audio_inbuf);
   free(session->audio_outbuf);
 
@@ -262,7 +276,7 @@ void session_audio_free(struct session_t *session) {
  *
  * returns 0 on success, -1 otherwise
  */
-int session_audio_deinit(struct session_t *session) {
+int session_audio_deinit(session_t *session) {
   /* free allocated buffers */
   session_audio_free(session);
   
@@ -282,7 +296,7 @@ int session_audio_deinit(struct session_t *session) {
  *
  * returns 0 on success, -1 otherwise
  */
-int session_isdn_deinit(struct session_t *session) {
+int session_isdn_deinit(session_t *session) {
   /* free allocated buffers */
   free(session->isdn_inbuf);
   free(session->isdn_outbuf);
@@ -318,7 +332,7 @@ int session_isdn_deinit(struct session_t *session) {
  *
  * returns 0 on success, -1 otherwise
  */
-int session_init(struct session_t *session,
+int session_init(session_t *session,
 		 char *audio_device_name_in,
 		 char *audio_device_name_out,
 		 char *msn, char *msns) {
@@ -363,6 +377,9 @@ int session_init(struct session_t *session,
   session->msn = strdup(msn);
   session->msns = strdup(msns);
 
+  session->from = strdup("");
+  session->to = strdup("");
+
   settings_options_read(session); /* override defaults analyzing options file*/
 
   /* command line configurable parameters: set to hard coded defaults
@@ -424,7 +441,8 @@ void free_g_list_element(gpointer data, gpointer user_data _U_) {
  *
  * returns 0 on success, -1 otherwise
  */
-int session_deinit(struct session_t *session) {
+int session_deinit(session_t *session) {
+  int i;
 
   /* deinit server functionality */
   if (server_deinit(session) < 0) return -1;
@@ -450,13 +468,21 @@ int session_deinit(struct session_t *session) {
   free(session->msn);
   free(session->msns);
 
+  /* clean up rest */
+  for (i = 0; i < 4; i++) {
+    free(session->preset_names[i]);
+    free(session->preset_numbers[i]);
+  }
+  free(session->from);
+  free(session->to);
+
   return 0;
 }
 
 /*
  * set up handlers for audio/ISDN input
  */
-void session_io_handlers_start(struct session_t *session) {
+void session_io_handlers_start(session_t *session) {
   if (!(session->option_release_devices &&
 	(session->state == STATE_READY ||
 	 session->state == STATE_RINGING_QUIET))) {
@@ -480,7 +506,7 @@ void session_io_handlers_start(struct session_t *session) {
 /*
  * remove handlers for audio/ISDN input
  */
-void session_io_handlers_stop(struct session_t *session) {
+void session_io_handlers_stop(session_t *session) {
   if (!(session->option_release_devices &&
 	(session->state == STATE_READY ||
 	 session->state == STATE_RINGING_QUIET))) {
@@ -501,7 +527,7 @@ void session_io_handlers_stop(struct session_t *session) {
  *
  * returns 0 on success, -1 on error
  */
-int session_reset_audio(struct session_t *session) {
+int session_reset_audio(session_t *session) {
   int result = 0;
   
   if (!(session->option_release_devices &&
@@ -527,7 +553,7 @@ int session_reset_audio(struct session_t *session) {
  * handlers in functions like isdn_setMSN or isdn_hangup where "OK\r\n" will
  * be checked for
  */
-void session_new_modem_string_callback(struct session_t *session) {
+void session_new_modem_string_callback(session_t *session) {
   /* modem string is in session->isdn_inbuf */
   if (debug)
     /* new line is in isdn_inbuf */
@@ -550,7 +576,7 @@ void session_new_modem_string_callback(struct session_t *session) {
  *         non-compleded lines are 1-terminated there
  *       * completed lines are actually terminated by "\r\n\0"
  */
-static int isdn_try_read_line(struct session_t *session) {
+static int isdn_try_read_line(session_t *session) {
   int total = 0; /* number of bytes read in this call */
   struct timeval tv;
   fd_set fds;
@@ -593,7 +619,7 @@ static int isdn_try_read_line(struct session_t *session) {
 }
 
 /* do some initialization of full duplex conversation mode */
-void session_init_conversation(struct session_t *session) {
+void session_init_conversation(session_t *session) {
   session->samples_in = 0;
   session->samples_out = 0;
 
@@ -613,7 +639,7 @@ void session_init_conversation(struct session_t *session) {
  *
  * return: self_hangup: 1 == we hangup, 0 == other side hung up
  */
-void session_deinit_conversation(struct session_t *session, int self_hangup) {
+void session_deinit_conversation(session_t *session, int self_hangup) {
   if (session->option_record) {
     recording_write(session->recorder, session->rec_buf_local,
                     session->rec_buf_local_index, RECORDING_LOCAL);
@@ -650,7 +676,7 @@ void session_deinit_conversation(struct session_t *session, int self_hangup) {
  * will be called directly after getting VCON from ttyI
  * includes state transition
  */
-void session_start_conversation(struct session_t *session) {
+void session_start_conversation(session_t *session) {
   char *digits; /* time in digits form */
   
   if (isdn_set_full_duplex(session->isdn_fd)) {
@@ -697,7 +723,7 @@ void session_start_conversation(struct session_t *session) {
  */
 void session_effect_callback(gpointer data, gint fd _U_,
 			     GdkInputCondition condition _U_) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   int i;
   unsigned char x; /* generated ulaw sample */
   double seconds; /* position in seconds */
@@ -725,7 +751,7 @@ void session_effect_callback(gpointer data, gint fd _U_,
 /*
  * Start an effect (effect_t) playing on the sound device
  */
-void session_effect_start(struct session_t *session, enum effect_t kind) {
+void session_effect_start(session_t *session, enum effect_t kind) {
   session->effect_tag = gdk_input_add(session->audio_fd_out,
 				      GDK_INPUT_WRITE,
 				      session_effect_callback,
@@ -738,7 +764,7 @@ void session_effect_start(struct session_t *session, enum effect_t kind) {
 /*
  * Reset sound device and unset callback
  */
-void session_effect_stop(struct session_t *session) {
+void session_effect_stop(session_t *session) {
   if (session->effect != EFFECT_NONE) { /* stop only if already playing */
     gdk_input_remove(session->effect_tag);
     session->effect = EFFECT_NONE;
@@ -752,7 +778,7 @@ void session_effect_stop(struct session_t *session) {
  * Sets status bar audio state (e.g. "AUDIO OFF")
  * hide if note is ""
  */
-void session_audio_notify(struct session_t *session, char *note) {
+void session_audio_notify(session_t *session, char *note) {
   gtk_widget_hide(session->audio_warning);
   if (*note) {
     gtk_statusbar_pop(GTK_STATUSBAR(session->audio_warning),
@@ -768,7 +794,7 @@ void session_audio_notify(struct session_t *session, char *note) {
  *
  * returns 0 on success, -1 otherwise (can't open audio device)
  */
-int session_set_state(struct session_t *session, enum state_t state) {
+int session_set_state(session_t *session, enum state_t state) {
   int result = 0;
 
   /* open / close audio when needed, set state */
@@ -804,7 +830,7 @@ int session_set_state(struct session_t *session, enum state_t state) {
   case STATE_RINGING:
     if (session->option_popup) {
 /* XXX: macros yet to define */
-#ifdef HAVE_GTK_WINDOW_PRESENT
+#if GTK_MAJOR_VERSION >= 2
       gtk_window_present(GTK_WINDOW(session->main_window));
 #else
       gdk_window_raise(session->main_window->window);
@@ -816,7 +842,7 @@ int session_set_state(struct session_t *session, enum state_t state) {
     break;
   case STATE_RINGING_QUIET:
     if (session->option_popup) {
-#ifdef HAVE_GTK_WINDOW_PRESENT
+#if GTK_MAJOR_VERSION >= 2
       gtk_window_present(GTK_WINDOW(session->main_window));
 #else
       gdk_window_raise(session->main_window->window);
@@ -881,13 +907,13 @@ int session_set_state(struct session_t *session, enum state_t state) {
 /*
  * callback for gdk on isdn input
  *
- * input: data:      session (struct session_t *)
+ * input: data:      session (session_t *)
  *        fd:        file descriptor where we got the input from
  *        condition: will be GDK_INPUT_READ in this case
  */
 void gdk_handle_isdn_input(gpointer data, gint fd _U_,
 			   GdkInputCondition condition _U_) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   char *temp;
 
   switch (session->state) {
@@ -897,15 +923,12 @@ void gdk_handle_isdn_input(gpointer data, gint fd _U_,
 	session->isdn_inbuf[session->isdn_inbuf_len - 2] = 0;
 	/* caller id update */
 	session->ring_time = time(NULL);
-	cid_add_line(session, CALL_IN, NULL, &session->isdn_inbuf[5]);
 
-	/* execute command if given */
-	if (session->exec_on_incoming) {
-	  temp = strdup(session->exec_on_incoming);
-	  substitute(&temp, "%n", &session->isdn_inbuf[5]);
-	  execute(temp);
-	  free(temp);
-	}
+	/* save callee's number */
+	free(session->to);
+	session->to = strdup(&session->isdn_inbuf[5]);
+	
+	cid_add_line(session, CALL_IN, NULL, session->to);
 
 	if (session_set_state(session, STATE_RINGING))
 	  session_set_state(session, STATE_RINGING_QUIET);
@@ -938,13 +961,28 @@ void gdk_handle_isdn_input(gpointer data, gint fd _U_,
       } else if (strstr(session->isdn_inbuf, "CALLER NUMBER: ")) {
 	/* got Caller ID */
 	session->isdn_inbuf[session->isdn_inbuf_len - 2] = 0;
+	
+	/* save caller's number */
+	free(session->from);
+	session->from = strdup(&session->isdn_inbuf[15]);
+
 	/* complete from field */
-	cid_set_from(session, &session->isdn_inbuf[15]);
+	cid_set_from(session, session->from);
+
+	/* execute command if given */
+	if (session->exec_on_incoming) {
+	  temp = strdup(session->exec_on_incoming);
+	  substitute(&temp, "%n", session->to);
+	  substitute(&temp, "%s", session->from);
+	  execute(temp);
+	  free(temp);
+	}
+	
       } else if (strstr(session->isdn_inbuf, "RUNG\r\n")) {
 	/* caller giving up */
 	session_set_state(session, STATE_READY);
 	cid_set_duration(session, "(RUNG)");
-	cid_mark_row(session, session->cid_num - 1, 1);
+	cid_mark_row(session, session->cid_num - 1, TRUE);
       } else { /* got some other modem answer string while it rings */
       }
     }
@@ -979,13 +1017,13 @@ void gdk_handle_isdn_input(gpointer data, gint fd _U_,
 /*
  * callback for gdk on audio isdn input
  *
- * input: data:      session (struct session_t *)
+ * input: data:      session (session_t *)
  *        fd:        file descriptor where we got the input from
  *        condition: will be GDK_INPUT_READ in this case
  */
 void gdk_handle_audio_input(gpointer data, gint fd _U_,
 			    GdkInputCondition condition _U_) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
 
   switch (session->state) {
   case STATE_READY: /* we are in command mode */
@@ -1059,7 +1097,7 @@ void gdk_handle_audio_input(gpointer data, gint fd _U_,
  * initiates dialing to specified number
  * -> changes contents of dial entry and simulates pick up button
  */
-void session_make_call(struct session_t *session, char *number) {
+void session_make_call(session_t *session, char *number) {
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(session->dial_number_box)->entry),
 		     number);
   gtk_button_clicked(GTK_BUTTON(session->pick_up_button));
@@ -1069,10 +1107,10 @@ void session_make_call(struct session_t *session, char *number) {
  * callback for GTK on pick up button clicked
  *
  * input: widget: the button
- *        data:   will be a (struct session_t *)
+ *        data:   will be a (session_t *)
  */
 void gtk_handle_pick_up_button(GtkWidget *widget _U_, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
   char *s; /* the modem dial string */
   char *number; /* the number to dial "inside" gtk (entry) */
   char *clear_number; /* number after un_vanity() */
@@ -1103,6 +1141,12 @@ void gtk_handle_pick_up_button(GtkWidget *widget _U_, gpointer data) {
 	  /* caller id update */
 	  session->ring_time = time(NULL);
 	  cid_add_line(session, CALL_OUT, session->msn, clear_number);
+	  
+	  /* save caller's and callee's number */
+	  free(session->from);
+	  session->from = strdup(session->msn);
+	  free(session->to);
+	  session->to = strdup(clear_number);
 	}
       } else {
 	show_audio_error_dialog();
@@ -1142,10 +1186,10 @@ void gtk_handle_pick_up_button(GtkWidget *widget _U_, gpointer data) {
  * callback for GTK on hang up button clicked, !!! also called on exit !!!
  *
  * input: widget: the button, NULL when called directly (on exit)
- *        data:   will be a (struct session_t *)
+ *        data:   will be a (session_t *)
  */
 void gtk_handle_hang_up_button(GtkWidget *widget _U_, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
+  session_t *session = (session_t *) data;
 
   switch (session->state) {
   case STATE_READY: /* we are already in command mode */
@@ -1184,7 +1228,7 @@ void gtk_handle_hang_up_button(GtkWidget *widget _U_, gpointer data) {
  * (session->dial_number_history_maxlen)
  * -> and redisplay in session->dial_number_box
  */
-static void session_history_normalize(struct session_t *session) {
+static void session_history_normalize(session_t *session) {
   /* cut size if needed */
   while (g_list_length(session->dial_number_history) >
 	 session->dial_number_history_maxlen + 1) {
@@ -1204,7 +1248,7 @@ static void session_history_normalize(struct session_t *session) {
  *
  * number will be copied, so caller has to care about it's associated memory
  */
-void session_history_add(struct session_t *session, char *number) {
+void session_history_add(session_t *session, char *number) {
   char *temp = strdup(number);
 
   session->dial_number_history = g_list_insert(
@@ -1215,7 +1259,7 @@ void session_history_add(struct session_t *session, char *number) {
 /*
  * like session_history_add but _appending_ number to list
  */
-void session_history_append(struct session_t *session, char *number) {
+void session_history_append(session_t *session, char *number) {
   char *temp = strdup(number);
 
   session->dial_number_history = g_list_append(
