@@ -5,6 +5,20 @@
  *
  * Copyright 2002, 2003 Roland Stigge
  *
+ * ANT is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ANT is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ANT; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
  *
  *
  * To add a new option, do the following:
@@ -40,7 +54,7 @@ int callerid_parse(void *session);
  *
  * used as callback from yyparse()
  */
-void settings_option_set(struct session_t *session, char *option, char *value) {
+void settings_option_set(session_t *session, char *option, char *value) {
   int i_value = -1; /* set value to 0 or 1 if appropriate */
 
   if (!strcmp(value, "0") || !strcasecmp(value, "false") ||
@@ -175,7 +189,7 @@ void settings_option_set(struct session_t *session, char *option, char *value) {
  * Here, we just parse the config file and _change_ options. the defaults
  * will be all set at init time
  */
-void settings_options_read(struct session_t *session) {
+void settings_options_read(session_t *session) {
   char *homedir;
   char *filename;
 
@@ -205,7 +219,7 @@ void settings_options_read(struct session_t *session) {
 /*
  * write options to options file (in dotfile directory)
  */
-void settings_options_write(struct session_t *session) {
+void settings_options_write(session_t *session) {
   int i;
   char *homedir;
   char *filename;
@@ -324,7 +338,7 @@ void settings_options_write(struct session_t *session) {
 }
 
 /* Read history file and set dial combo box */
-void settings_history_read(struct session_t *session) {
+void settings_history_read(session_t *session) {
   char *homedir;
   char *filename;
   FILE *f;
@@ -384,7 +398,7 @@ static void settings_history_write_line(gpointer s, gpointer f) {
 /*
  * write out dial history file
  */
-void settings_history_write(struct session_t *session) {
+void settings_history_write(session_t *session) {
   char *homedir;
   char *filename;
   FILE *f;
@@ -419,7 +433,7 @@ void settings_history_write(struct session_t *session) {
 }
 
 /* Read callerid history file */
-void settings_callerid_read(struct session_t *session) {
+void settings_callerid_read(session_t *session) {
   char *homedir;
   char *filename;
 
@@ -450,7 +464,7 @@ void settings_callerid_read(struct session_t *session) {
 /*
  * write out callerid history file
  */
-void settings_callerid_write(struct session_t *session) {
+void settings_callerid_write(session_t *session) {
   char *homedir;
   char *filename;
   FILE *f;
@@ -479,13 +493,16 @@ void settings_callerid_write(struct session_t *session) {
   if ((f = fopen(filename, "w"))) {
     for (akt_line = 0; akt_line < session->cid_num; akt_line++)
     {
-      gtk_clist_get_text (GTK_CLIST(session->cid_list), akt_line, 0, &date);
-      gtk_clist_get_pixtext (GTK_CLIST(session->cid_list), akt_line, 1, &type,
-			     NULL, NULL, NULL);
-      gtk_clist_get_text (GTK_CLIST(session->cid_list), akt_line, 2, &from);
-      gtk_clist_get_text (GTK_CLIST(session->cid_list), akt_line, 3, &to);
-      gtk_clist_get_text (GTK_CLIST(session->cid_list), akt_line, 4,
-			  &duration);
+      gtk_clist_get_text(GTK_CLIST(session->cid_list), akt_line,
+	                 CID_COL_TIME, &date);
+      gtk_clist_get_pixtext(GTK_CLIST(session->cid_list), akt_line,
+	                    CID_COL_TYPE, &type, NULL, NULL, NULL);
+      gtk_clist_get_text(GTK_CLIST(session->cid_list), akt_line,
+	                 CID_COL_FROM, &from);
+      gtk_clist_get_text(GTK_CLIST(session->cid_list), akt_line,
+	                 CID_COL_TO, &to);
+      gtk_clist_get_text(GTK_CLIST(session->cid_list), akt_line,
+	                 CID_COL_DURATION, &duration);
       
       fprintf (f, "%-19s|%-3s|%-20s|%-20s|%-10s\n",
 	       date, type, from, to, duration);
